# sage_setup: distribution = sagemath-environment
r"""
Features for testing the presence of ``singular`` and the SageMath interfaces to it
"""

# *****************************************************************************
#       Copyright (C) 2022-2023 Matthias Koeppe
#
#  Distributed under the terms of the GNU General Public License (GPL)
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#                  https://www.gnu.org/licenses/
# *****************************************************************************

from . import Executable, PythonModule
from .join_feature import JoinFeature
from .sagemath import sage__libs__singular
from sage.env import SINGULAR_BIN


class Singular(Executable):
    r"""
    A :class:`~sage.features.Feature` describing the presence of the :ref:`singular <spkg_singular>` executable.

    .. SEEALSO::

        :class:`Feature sage.libs.singular <~sage.features.sagemath.sage__libs__singular>`

    EXAMPLES::

        sage: from sage.features.singular import Singular
        sage: Singular().is_present()                                                   # needs singular
        FeatureTestResult('singular', True)
    """
    def __init__(self):
        r"""
        TESTS::

            sage: from sage.features.singular import Singular
            sage: isinstance(Singular(), Singular)
            True
        """
        Executable.__init__(self, "singular", SINGULAR_BIN,
                            spkg='singular', type='standard')


def all_features():
    return [Singular()]



#!/usr/bin/env python3

########################################################################
# Originally based on a script by Andrew Dalke:
#    http://projects.scipy.org/pipermail/numpy-discussion/2008-July/035415.html
#
# 2012: Total rewrite by Volker Braun
########################################################################


########################################################################
#       Copyright (C) 2012 Volker Braun <vbraun.name@gmail.com>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#
#                  https://www.gnu.org/licenses/
########################################################################
import sys
import os
import time
import gc
import warnings

# Ignore collections.abc warnings, there are a lot of them but they are
# harmless. These warnings are also disabled in src/sage/all.py.
warnings.filterwarnings('ignore', category=DeprecationWarning,
                        message='.*collections[.]abc.*')

cmdline_args = sys.argv[2:]
have_cmdline_args = bool(cmdline_args)

direct_children_time = 0
import_counter = 0
parent = None
index_to_parent = dict()
all_modules = dict()

DEFAULT_LEVEL = 0


def new_import(name, globals={}, locals={}, fromlist=[], level=DEFAULT_LEVEL):
    """
    The new import function

    Note that ``name`` is not unique, it can be `sage.foo.bar` or `bar`.
    """
    global all_modules, import_counter, parent, direct_children_time
    old_direct_children_time = direct_children_time
    direct_children_time = 0
    old_parent = parent
    parent = this_import_counter = import_counter
    import_counter += 1
    t1 = time.time()
    module = old_import(name, globals, locals, fromlist, level)
    t2 = time.time()
    parent = old_parent
    elapsed_time = t2 - t1
    module_time = elapsed_time - direct_children_time
    direct_children_time = old_direct_children_time + elapsed_time
    index_to_parent[this_import_counter] = module
    data = all_modules.get(module, None)
    if data is not None:
        data['parents'].append(parent)
        data['import_names'].add(name)
        data['cumulative_time'] += elapsed_time
        data['time'] += module_time
        return module
    data = {'cumulative_time': elapsed_time,
            'time': module_time,
            'import_names': set([name]),
            'parents': [parent]}
    all_modules[module] = data
    return module


old_import = __builtins__.__import__
__builtins__.__import__ = new_import
gc.disable()
from sage.all import *
gc.enable()
__builtins__.__import__ = old_import

for data in all_modules.values():
    data['parents'] = set(index_to_parent.get(i, None)
                          for i in data['parents'])


module_by_speed = sorted(((data['time'], module, data)
                          for module, data in all_modules.items()),
                         key=lambda x: x[0])


def print_separator():
    print('=' * 72)


def print_headline(line):
    print('=={0:=<68}=='.format(' ' + line + ' '))


width = 10
fmt_header = '{0:>' + str(width) + '} {1:>' + str(width) + '} {2:>' + str(width) + '}  {3}'
fmt_number = '{0:>' + str(width) + '.3f} {1:>' + str(width) + '.3f} {2:>' + str(width) + '}  {3}'


def print_table(module_list, limit):
    global fmt_header, fmt_number
    print(fmt_header.format('exclude/ms', 'include/ms', '#parents', 'module name'))
    for t, module, data in module_list[-limit:]:
        print(fmt_number.format(1000 * t, 1000 * data['cumulative_time'],
                                len(data['parents']), module.__name__))


def guess_module_name(src):
    module = []
    src, ext = os.path.splitext(src)
    while src and src != '/':
        head, tail = os.path.split(os.path.abspath(src))
        if (tail == 'src' or any(os.path.exists(os.path.join(head, tail, f))
                                 for f in ('setup.py', 'pyproject.toml'))):
            return '.'.join(module)
        module.insert(0, tail)
        src = head
    return None


if not have_cmdline_args:
    print('== Slowest module imports (excluding / including children) ==')
    print_table(module_by_speed, 50)
    print('Total time (sum over exclusive time): {:.3f}ms'.format(1000 * sum(data[0] for data in module_by_speed)))
    print('Use sage -startuptime <module_name|file_name>... to get more details about specific modules.')
else:
    for module_arg in cmdline_args:
        matching_modules = [m for m in all_modules if m.__name__ == module_arg]
        if not matching_modules:
            if '/' in module_arg or any(module_arg.endswith(ext) for ext in ('.py', '.pyx')) or os.path.isdir(module_arg):
                file_name = module_arg
                module_arg = guess_module_name(file_name)
                if not module_arg:
                    print('Warning: "' + file_name + '" does not appear to be a Python module source file or package directory.')
                    continue
                else:
                    matching_modules = [m for m in all_modules if m.__name__.startswith(module_arg)]
            else:
                matching_modules = [m for m in all_modules if m.__name__.endswith(module_arg)]
        if not matching_modules:
            print('Warning: No modules loaded at startup correspond to {}'.format(module_arg))
        for module_name in matching_modules:
            parents = all_modules[module_name]['parents']
            print()
            print_separator()
            print_headline('Slowest modules importing {0}'.format(module_name.__name__))
            print_table([m for m in module_by_speed if m[1] in parents], 10)
            print()
            print_headline('Slowest modules imported by {0}'.format(module_name.__name__))
            print_table([m for m in module_by_speed if module_name in m[2]['parents']], 10)
            print()
            data = all_modules[module_name]
            print_headline('module ' + module_name.__name__)
            print('Time to import:  {0:.3f}ms'.format(1000 * data['time']))
            print('Cumulative time: {0:.3f}ms'.format(1000 * data['cumulative_time']))
            print('Names: {0}'.format(', '.join(data['import_names'])))
            print('File: {0}'.format(module_name.__file__))



# sage_setup: distribution = sagemath-environment
r"""
Feature for testing the presence of ``cddlib``
"""

# *****************************************************************************
#       Copyright (C) 2022 Matthias Koeppe
#
#  Distributed under the terms of the GNU General Public License (GPL)
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#                  https://www.gnu.org/licenses/
# *****************************************************************************

from . import Executable


class CddExecutable(Executable):
    r"""
    A :class:`~sage.features.Feature` describing the presence of an executable
    which comes as a part of :ref:`cddlib <spkg_cddlib>`.

    EXAMPLES::

        sage: from sage.features.cddlib import CddExecutable
        sage: CddExecutable().is_present()
        FeatureTestResult('cddexec_gmp', True)
    """
    def __init__(self, name='cddexec_gmp'):
        r"""
        TESTS::

            sage: from sage.features.cddlib import CddExecutable
            sage: isinstance(CddExecutable(), CddExecutable)
            True
        """
        Executable.__init__(self, name=name, executable=name, spkg='cddlib',
                            url='https://github.com/cddlib/cddlib', type='standard')



# sage_setup: distribution = sagemath-environment
r"""
Feature for testing the presence of ``kenzo``
"""

# ****************************************************************************
#       Copyright (C) 2020 Travis Scrimshaw
#                     2021 Matthias Koeppe
#                     2021 Michael Orlitzky
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#                  https://www.gnu.org/licenses/
# ****************************************************************************


from . import Feature, FeatureTestResult


class Kenzo(Feature):
    r"""
    A :class:`~sage.features.Feature` describing the presence of :ref:`Kenzo <spkg_kenzo>`.

    EXAMPLES::

        sage: from sage.features.kenzo import Kenzo
        sage: Kenzo().is_present()  # optional - kenzo
        FeatureTestResult('kenzo', True)
    """
    def __init__(self):
        r"""
        TESTS::

            sage: from sage.features.kenzo import Kenzo
            sage: isinstance(Kenzo(), Kenzo)
            True
        """
        Feature.__init__(self, name='kenzo', spkg='kenzo',
                         url='https://github.com/miguelmarco/kenzo/')

    def _is_present(self):
        r"""
        Check whether Kenzo is installed and works.

        EXAMPLES::

            sage: from sage.features.kenzo import Kenzo
            sage: Kenzo()._is_present()  # optional - kenzo
            FeatureTestResult('kenzo', True)
        """
        try:
            from sage.libs.ecl import ecl_eval
        except ImportError:
            return FeatureTestResult(self, False, reason="sage.libs.ecl is not available")
        # Redirection of ECL and Maxima stdout to /dev/null
        # This is also done in the Maxima library, but we
        # also do it here for redundancy.
        ecl_eval(r"""(defparameter *dev-null* (make-two-way-stream
                      (make-concatenated-stream) (make-broadcast-stream)))""")
        ecl_eval("(setf original-standard-output *standard-output*)")
        ecl_eval("(setf *standard-output* *dev-null*)")

        try:
            from sage.env import KENZO_FAS
            if KENZO_FAS:
                ecl_eval("(require :kenzo \"{}\")".format(KENZO_FAS))
            else:
                ecl_eval("(require :kenzo)")

        except RuntimeError:
            return FeatureTestResult(self, False, reason="Unable to make ECL require kenzo")
        return FeatureTestResult(self, True)


def all_features():
    return [Kenzo()]



# Sage version information for Python scripts
# This file is auto-generated by the update-version script, do not edit!
version = '10.7.rc1'
date = '2025-08-05'
banner = 'SageMath version 10.7.rc1, Release Date: 2025-08-05'



# sage_setup: distribution = sagemath-environment
r"""
Check for various standard packages (for modularized distributions)

These features are provided by standard packages in the Sage distribution.
"""

# *****************************************************************************
#       Copyright (C) 2023 Matthias Koeppe
#
#  Distributed under the terms of the GNU General Public License (GPL)
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#                  https://www.gnu.org/licenses/
# *****************************************************************************

from . import PythonModule
from .join_feature import JoinFeature


def all_features():
    return [PythonModule('cvxopt', spkg='cvxopt', type='standard'),
            PythonModule('fpylll', spkg='fpylll', type='standard'),
            JoinFeature('ipython', (PythonModule('IPython'),), spkg='ipython', type='standard'),
            JoinFeature('lrcalc_python', (PythonModule('lrcalc'),), spkg='lrcalc_python', type='standard'),
            PythonModule('mpmath', spkg='mpmath', type='standard'),
            PythonModule('networkx', spkg='networkx', type='standard'),
            PythonModule('numpy', spkg='numpy', type='standard'),
            PythonModule('pexpect', spkg='pexpect', type='standard'),
            JoinFeature('pillow', (PythonModule('PIL'),), spkg='pillow', type='standard'),
            JoinFeature('pplpy', (PythonModule('ppl'),), spkg='pplpy', type='standard'),
            PythonModule('primecountpy', spkg='primecountpy', type='standard'),
            PythonModule('ptyprocess', spkg='ptyprocess', type='standard'),
            PythonModule('pyparsing', spkg='pyparsing', type='standard'),
            PythonModule('requests', spkg='requests', type='standard'),
            PythonModule('rpy2', spkg='rpy2', type='standard'),
            PythonModule('scipy', spkg='scipy', type='standard'),
            PythonModule('sympy', spkg='sympy', type='standard')]



# sage_setup: distribution = sagemath-environment
r"""
Features for testing the presence of ``4ti2``
"""

from . import Executable
from .join_feature import JoinFeature


class FourTi2Executable(Executable):
    r"""
    A :class:`~sage.features.Feature` for the :ref:`4ti2 <spkg_4ti2>` executables.
    """
    def __init__(self, name):
        r"""
        TESTS::

            sage: from sage.features.four_ti_2 import FourTi2Executable
            sage: isinstance(FourTi2Executable('hilbert'), FourTi2Executable)
            True
        """
        from sage.env import SAGE_ENV
        Executable.__init__(self,
                            name="4ti2-" + name,
                            executable=SAGE_ENV.get("FOURTITWO_" + name.upper(), None) or name,
                            spkg='4ti2')


class FourTi2(JoinFeature):
    r"""
    A :class:`~sage.features.Feature` describing the presence of all :ref:`4ti2 <spkg_4ti2>` executables.

    EXAMPLES::

        sage: from sage.features.four_ti_2 import FourTi2
        sage: FourTi2().is_present()  # optional - 4ti2
        FeatureTestResult('4ti2', True)
    """
    def __init__(self):
        r"""
        TESTS::

            sage: from sage.features.four_ti_2 import FourTi2
            sage: isinstance(FourTi2(), FourTi2)
            True
        """
        JoinFeature.__init__(self, '4ti2',
                             [FourTi2Executable(x)
                              # same list is tested in build/pkgs/4ti2/spkg-configure.m4
                              for x in ('hilbert', 'markov', 'graver', 'zsolve', 'qsolve',
                                        'rays', 'ppi', 'circuits', 'groebner')])


def all_features():
    return [FourTi2()]



# sage_setup: distribution = sagemath-environment
r"""
Features for testing the presence of ``fricas``
"""

# *****************************************************************************
#       Copyright (C) 2023 Dima Pasechnik
#
#  Distributed under the terms of the GNU General Public License (GPL)
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#                  https://www.gnu.org/licenses/
# *****************************************************************************

import subprocess
from . import Executable, FeatureTestResult
from packaging.version import Version


class FriCAS(Executable):
    r"""
    A :class:`~sage.features.Feature` which checks for the :ref:`fricas <fricas>` binary.

    EXAMPLES::

        sage: from sage.features.fricas import FriCAS
        sage: FriCAS().is_present()  # optional - fricas
        FeatureTestResult('fricas', True)
    """
    MINIMUM_VERSION = "1.3.8"

    def __init__(self):
        r"""
        TESTS::

            sage: from sage.features.fricas import FriCAS
            sage: isinstance(FriCAS(), FriCAS)
            True
        """
        Executable.__init__(self, name='fricas', spkg='fricas',
                            executable='fricas',
                            url='https://fricas.github.io')

    def get_version(self):
        r"""
        Retrieve the installed FriCAS version

        EXAMPLES::
            sage: from sage.features.fricas import FriCAS
            sage: FriCAS().get_version() # optional - fricas
            '1.3...'
        """
        try:
            output = subprocess.check_output(['fricas', '--version'], stderr=subprocess.STDOUT)
            version_line = output.decode('utf-8').strip()
            version = version_line.split()[1]
            return version
        except subprocess.CalledProcessError:
            return None

    def is_functional(self):
        r"""
        Check whether ``fricas`` works on trivial input.

        EXAMPLES::

            sage: from sage.features.fricas import FriCAS
            sage: FriCAS().is_functional()  # optional - fricas
            FeatureTestResult('fricas', True)
        """
        command = ['fricas -nosman -eval ")quit"']
        try:
            lines = subprocess.check_output(command, stderr=subprocess.STDOUT, shell=True)
        except subprocess.CalledProcessError as e:
            return FeatureTestResult(self, False,
                                     reason="Call `{command}` failed with exit code {e.returncode}".format(command=" ".join(command), e=e))

        expected = b"FriCAS"
        if lines.find(expected) == -1:
            return FeatureTestResult(self, False,
                                     reason="Call `{command}` did not produce output which contains `{expected}`".format(command=" ".join(command),
                                                                                                                         expected=expected))
        version = self.get_version()
        if version is None:
            return FeatureTestResult(self, False,
                                     reason="Could not determine FriCAS version")

        try:
            if Version(version) < Version(self.MINIMUM_VERSION):
                return FeatureTestResult(self, False, reason=f"FriCAS version {version} is too old; minimum required is {self.MINIMUM_VERSION}")
            return FeatureTestResult(self, True)
        except ValueError:
            return FeatureTestResult(self, False, reason="Invalid Version Format")


def all_features():
    return [FriCAS()]
